#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""FAL command for LOG Extention."""

from iobase.xmldata import (XmlConfNode,)
from iobase.exception import (InvalidArgError, IoBaseException, NotFoundError)
from ionas.request import Entry

from pal_info import (PAL_DEVCTRL_PATH, PAL_LOGEXT_PATH, PAL_CONFFILE_PATH,
                        PAL_SAMBA_PATH)
import libfal.falcmd as falcmd
import libfal.util as util
from fal_info import (ReqTypes, Results)
from fal_config import (STOR_IF_TO_VOL, VOL_INFO)

from pkgaddon.fal_logext_config import (CONFIG_FILE, REGISTERED_KEY_FILE,
                                        USB_KEY_PORT_NO_LIST,
                                        INVALID_USB_MEMORY_ERROR,
                                        ALREADY_REGISTERED_ERROR)


LOGEXT_NAME = "logext"

APPLY_TAG_LIST = ["restrictionlist", "eventlogconf", "accesslogconf",
                    "share", "alertlist"]
DELETE_INFO_LIST_MAP = {
        "accesslogconf": [("sharelist", "*")],
        "share": [
                ("permission/detailpermission/readwritemember/userlist", "*"),
                ("permission/detailpermission/readwritemember/grouplist", "*"),
                ("permission/detailpermission/readonlymember/userlist", "*"),
                ("permission/detailpermission/readonlymember/grouplist", "*"),
            ],
        "alertlist": [(".", "*")],
    }

USB_KEY = "usb"
DEV_KEY = "dev"
DEVCTRL_QUERY_XML_TMPL = "<devctrl><device><name>%s</name></device></devctrl>"
DEVICE_USB_PATH = "device/usb"
KEYLIST_NAME = "keylist"
KEY_NAME = "key"
KEY_USB_PATH = "key/usb"
VID_NAME = "vid"
PID_NAME = "pid"
SERIAL_NAME = "serial"
UNSET1_VALUE = "__unset1__"
UNSET2_VALUE = "__unset2__"
RESTRICTION_LOGSHARE_XPATH = \
        """restrictionlist/restriction[@type="logshare"]"""
RESTRICTION_LOGSETTING_XPATH = \
        """restrictionlist/restriction[@type="logsetting"]"""
SHARE_CATEGORY = "share"
SHARE_NAME = "share"
SAMBA_NAME = "samaba"
PORT_ATTR = "port"
NOT_CONNECTED = ""


class RequestError(IoBaseException):
    """Request error class."""

    pass


class NoMoreDelete(IoBaseException):
    """No more delete error class."""

    pass


class AlreadyRegistered(IoBaseException):
    """Already registered error class."""

    pass

class InvalidUsbKey(IoBaseException):
    """Invalid USB key error class."""

    pass


class FalLogExt(falcmd.FalCmd):
    """FAL LogExt command implementation class."""

    #
    # Internal methods
    #

    def _check_body_node(self, node, type=None):
        """Check if entry's body node is valid."""

        if node.get_node_name() != LOGEXT_NAME:
            raise InvalidArgError(node.get_node_name())

        if type == ReqTypes.UNREGISTER:
            try:
                node.get_node(KEY_USB_PATH)
            except:
                raise InvalidArgError("USB Key not Found")

    def _send_request(self, target, entry):
        """Send "entry" request to "target" PAL commands."""

        response_entry = self.send_entry(entry, target)
        if response_entry.get_result() != Results.OK:
            raise RequestError(response_entry.get_result())

        return response_entry.get()

    def _get_usb_disk_name(self, port_no):
        """Return the USB disk name responding to "port_no" port."""

        vol_name = STOR_IF_TO_VOL[USB_KEY][port_no]
        return VOL_INFO[vol_name][DEV_KEY]

    def _get_usb_info(self, name):
        """Return the USB information whose device name is "name"."""

        response = self._send_request(PAL_DEVCTRL_PATH,
                    Entry(ReqTypes.QUERY, DEVCTRL_QUERY_XML_TMPL % (name,)))
        return response.get_node(DEVICE_USB_PATH)

    def _get_usb_key_device_info(self):
        """Return the connected USB Key device information."""

        for port_no in USB_KEY_PORT_NO_LIST:
            try:
                return self._get_usb_info(self._get_usb_disk_name(port_no))
            except:
                self.warning("Not found : %s" % (port_no,))
                # Ignore error
                continue

        raise NotFoundError("USB Key")

    def _is_valid_usb_key(self, device_info):
        """Check if USB Key device is valid."""

        if (device_info.get_value(VID_NAME) == "" or
            device_info.get_value(PID_NAME) == "" or
            device_info.get_value(SERIAL_NAME) == ""):

            return False

        return True

    def _load_registered_key_list(self):
        """Load and return the XML node of registered key list."""

        try:
            keylist = util.load_config(REGISTERED_KEY_FILE)
        except:
            keylist = XmlConfNode(KEYLIST_NAME)

        return keylist

    def _save_registered_key_list(self, keylist):
        """Save the "keylist" XML node."""

        util.save_config(REGISTERED_KEY_FILE, keylist)

    def _is_same_device(self, device_info, other):
        """Check if a "device_info" device is same as "other" device."""

        if (device_info.get_value(VID_NAME, UNSET1_VALUE) !=
                other.get_value(VID_NAME, UNSET2_VALUE)):

            return False

        if (device_info.get_value(PID_NAME, UNSET1_VALUE) !=
                other.get_value(PID_NAME, UNSET2_VALUE)):

            return False

        if (device_info.get_value(SERIAL_NAME, UNSET1_VALUE) !=
                other.get_value(SERIAL_NAME, UNSET2_VALUE)):

            return False

        return True

    def _is_usb_registered(self, usb_device_info, keylist):
        """Check if a "usb_device_info" device is registered in
           "keylist" XML data.
        """

        for usb in keylist.get_node_list(KEY_USB_PATH):
            if self._is_same_device(usb, usb_device_info):
                return True

        return False

    def _is_registered_usb_connected(self):
        """Check if a registered USB key is connected."""

        keylist = self._load_registered_key_list()
        for port_no in STOR_IF_TO_VOL[USB_KEY]:
            try:
                device_info = self._get_usb_info(
                                        self._get_usb_disk_name(port_no))
                if self._is_usb_registered(device_info, keylist):
                    return True
            except:
                # Ignore error
                continue

        return False

    def _is_setting_restricted(self, config, ignore_error=True):
        """Check if a setting is restricted."""

        try:
            return config.get_node(RESTRICTION_LOGSETTING_XPATH).is_enabled()
        except:
            if ignore_error:
                return False
            raise

    def _is_change_permitted(self, config):
        """Check if it is in condition that is permitted to change
           setting of a "logext" package.
        """

        try:
            if not self._is_setting_restricted(config):
                return True

            if self._is_registered_usb_connected():
                return True

        except:
            # Assume change is not permitted.
            pass

        return False

    def _is_share_restricted(self, config, ignore_error=True):
        """Check if a share is restricted."""

        try:
            return config.get_node(RESTRICTION_LOGSHARE_XPATH).is_enabled()
        except:
            if ignore_error:
                return False
            raise

    def _is_share_permitted(self, config):
        """Check if it is in condition that is permitted to change
           setting of a "logext" package.
        """

        try:
            if not self._is_share_restricted(config):
                return True

            if self._is_registered_usb_connected():
                return True

        except:
            # Assume change is not permitted.
            pass

        return False

    def _register_key(self):
        """Register a connected key device."""

        device_info = self._get_usb_key_device_info()
        if not self._is_valid_usb_key(device_info):
            raise InvalidUsbKey("Invalid USB Key");

        keylist = self._load_registered_key_list()

        if self._is_usb_registered(device_info, keylist):
            raise AlreadyRegistered("Target is already registered.")

        key = XmlConfNode(KEY_NAME)
        key.append_child(device_info)
        keylist.append_child(key)

        self._save_registered_key_list(keylist)

    def _unregister_usb_key(self, usb):
        """Unregister a USB key device."""

        keylist = self._load_registered_key_list()
        registered_usb_list = keylist.get_node_list(KEY_USB_PATH)
        if len(registered_usb_list) <= 1:
            raise NoMoreDelete("No more delete a USB key")

        for index, registered_usb in enumerate(registered_usb_list):
            if self._is_same_device(usb, registered_usb):
                keylist.delete_child(KEY_NAME, index=index)
                break
        self._save_registered_key_list(keylist)

    def _unregister_all_key(self):
        """Unregister a all key device."""

        self._save_registered_key_list(XmlConfNode(KEYLIST_NAME))

    def _update_registered_key_for_apply(self, config):
        """Try register a key device for apply request."""

        do_try_register_key = False

        if self._is_setting_restricted(config):
            do_try_register_key = True
        else:
            if self._is_share_restricted(config):
                do_try_register_key = True
            else:
                # setting and share restrictions are disabled.
                self._unregister_all_key()
                return

        if do_try_register_key:
            keylist = self._load_registered_key_list()
            if not len(keylist.get_node_list(KEY_USB_PATH)):
                try:
                    self._register_key()
                except InvalidUsbKey:
                    raise
                except NotFoundError:
                    raise
                except:
                    # Ignore exception. A key may be already registered.
                    pass

    def _try_deleting_old_setting(self, old_tag):
        """Try deleting old setting in "old_tag"."""

        del_info_list = DELETE_INFO_LIST_MAP.get(old_tag.get_node_name(), [])
        for del_info in del_info_list:
            old_tag.delete_nodes(del_info[0], del_info[1])

    def _update_share_setting(self, config):
        """Update the share setting by PAL."""

        if self._is_share_permitted(config):
            self._send_request(PAL_LOGEXT_PATH,
                    Entry(ReqTypes.ENABLE, LOGEXT_NAME))
        else:
            self._send_request(PAL_LOGEXT_PATH,
                    Entry(ReqTypes.DISABLE, LOGEXT_NAME))

        self._send_request(PAL_SAMBA_PATH, Entry(ReqTypes.UPDATE, SAMBA_NAME))

    def _request_to_pal_for_apply(self, config, is_share_changed):
        """Apply the share setting by PAL."""

        if not is_share_changed:
            self._send_request(PAL_LOGEXT_PATH,
                                Entry(ReqTypes.APPLY, LOGEXT_NAME))
            return

        util.servicectrl_suspend([SHARE_CATEGORY], [])

        try:
            req_body = XmlConfNode(LOGEXT_NAME)
            req_body.append_child(config.get_node(SHARE_NAME).get_clone())
            self._send_request(PAL_LOGEXT_PATH,
                                Entry(ReqTypes.APPLY, req_body))
            self._update_share_setting(config)
        finally:
            try:
                util.servicectrl_resume([SHARE_CATEGORY], [])
            except:
                # Ignore error
                pass

    def _update_keylist(self, keylist, usb_device_info, port_no):
        """Update keylist about a USB port which is connected."""

        for usb in keylist.get_node_list(KEY_USB_PATH):
            if self._is_same_device(usb, usb_device_info):
                usb.set_attrs({PORT_ATTR: str(port_no)})

    def _get_registered_key_list(self):
        """Return a regisered key list."""

        keylist = self._load_registered_key_list()

        for usb in keylist.get_node_list(KEY_USB_PATH):
            usb.set_attrs({PORT_ATTR: NOT_CONNECTED})

        for port_no in STOR_IF_TO_VOL[USB_KEY]:
            try:
                device_info = self._get_usb_info(
                                        self._get_usb_disk_name(port_no))
                self._update_keylist(keylist, device_info, port_no)
            except:
                # Ignore error
                continue

        return keylist

    #
    # Request handlers
    #

    def _apply_handler(self, entry):
        """Handle apply request."""

        self._check_body_node(entry.get(), ReqTypes.APPLY)
        try:
            config = util.load_config(CONFIG_FILE)
            new_config = entry.get()

            if not self._is_change_permitted(config):
                entry.set_result(Results.AUTH_ERROR)
                return entry

            is_share_changed = False
            for tag_name in APPLY_TAG_LIST:
                try:
                    new_target = new_config.get_node(tag_name)
                    try:
                        target = config.get_node(tag_name)
                        self._try_deleting_old_setting(target)
                        target.update(new_target, override=True)
                        if tag_name == SHARE_NAME:
                            is_share_changed = True
                    except NotFoundError:
                        config.append_child(new_target.get_clone())
                except:
                    # Not change setting about "tag_name" tag entry.
                    continue

            try:
                self._update_registered_key_for_apply(config)
            except NotFoundError:
                self.exception("Can't get USB key device info")
                entry.set_result(Results.DEVICE_NOT_FOUND)
                return entry
            except InvalidUsbKey:
                self.exception("Invalid USB Key")
                entry.set_result(INVALID_USB_MEMORY_ERROR)
                return entry

            self._request_to_pal_for_apply(config, is_share_changed)

            util.save_config(CONFIG_FILE, config)

            entry.set_result(Results.OK)
        except:
            self.exception("apply fail")
            entry.set_result(Results.APPLY_ERROR)

        return entry

    def _list_handler(self, entry):
        """Handle list request."""

        self._check_body_node(entry.get())
        entry.get().append_child(self._get_registered_key_list())
        entry.set_result(Results.OK)
        return entry

    def _register_handler(self, entry):
        """Handle register request."""

        self._check_body_node(entry.get())

        try:
            config = util.load_config(CONFIG_FILE)
            if not self._is_change_permitted(config):
                entry.set_result(Results.AUTH_ERROR)
                return entry

            self._register_key()
            entry.set_result(Results.OK)

        except NotFoundError:
            self.exception("Can't get USB key device info")
            entry.set_result(Results.DEVICE_NOT_FOUND)
        except InvalidUsbKey:
            self.exception("Invalid USB Key")
            entry.set_result(INVALID_USB_MEMORY_ERROR)
        except AlreadyRegistered:
            self.exception("Already registered")
            entry.set_result(ALREADY_REGISTERED_ERROR)
        except:
            self.exception("Add USB key info")
            entry.set_result(Results.ADD_ERROR)

        return entry

    def _unregister_handler(self, entry):
        """Handle unregister request."""

        self._check_body_node(entry.get(), ReqTypes.UNREGISTER)

        try:
            config = util.load_config(CONFIG_FILE)
            if not self._is_change_permitted(config):
                entry.set_result(Results.AUTH_ERROR)
                return entry

            self._unregister_usb_key(entry.get().get_node(KEY_USB_PATH))
            entry.set_result(Results.OK)
        except:
            self.exception("Delete USB key info")
            entry.set_result(Results.DELETE_ERROR)

        return entry

    def _update_handler(self, entry):
        """Handle update request."""

        self._check_body_node(entry.get())
        try:
            config = util.load_config(CONFIG_FILE)
            self._update_share_setting(config)
            entry.set_result(Results.OK)
        except:
            self.exception("update error")
            entry.set_result(Results.UPDATE_ERROR)

        return entry

    #
    # Constructor
    #

    def __init__(self, option=None):
        """Constructor."""

        falcmd.FalCmd.__init__(self, option=option)

        load_save_handler = lambda entry: self.send_entry_with_file(
            entry, PAL_CONFFILE_PATH, CONFIG_FILE)

        self.set_handler(ReqTypes.LOAD, load_save_handler)
        self.set_handler(ReqTypes.SAVE, load_save_handler)

        self.set_handler(ReqTypes.APPLY, self._apply_handler)
        self.set_handler(ReqTypes.LIST, self._list_handler)
        self.set_handler(ReqTypes.REGISTER, self._register_handler)
        self.set_handler(ReqTypes.UNREGISTER, self._unregister_handler)
        self.set_handler(ReqTypes.UPDATE, self._update_handler)


#
# Main
#

if __name__ == "__main__":
    FalLogExt(option=falcmd.FalCmdOption()).handle_request()
