#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""PAL command for LOG Extention."""


import os
import shutil

from iobase.exception import (InvalidArgError,)
from iobase.xmldata import (XmlConfNode,)
from iobase.cmdexec import (CmdExec, NULL)
from pal_info import (ReqTypes, Results)
import pal_config as config

import libpal.palcmd as palcmd

from pkgaddon.pal_logext_config import (LOGEXT_CONF_DIR, LOGEXT_CONF_TEMP_DIR,
                                        CONF_BASE_NAME, LOGEXTMGR_NAME,
                                        SEND_LOGEXT_CMD)


LOGEXT_NAME = "logext"
SHARE_NAME = "share"

NAME_PATH = "name"
VOLUME_PATH = "volume"
RO_SUFFIX = "ro"
RW_SUFFIX = "rw"
TRUE_TEXT = XmlConfNode.TRUE_TEXT
FALSE_TEXT = XmlConfNode.FALSE_TEXT
YES_VALUE = "yes"
NO_VALUE = "no"
DETAIL_PERMISSION_PATH = "permission/detailpermission"
READ_ONLY_MEMBER_PATH = DETAIL_PERMISSION_PATH + "/readonlymember"
READ_WRITE_MEMBER_PATH = DETAIL_PERMISSION_PATH + "/readwritemember"
USER_NAME_PATH = "/userlist/user/name"
GROUP_NAME_PATH = "/grouplist/group/name"
RO_USER_NAME_PATH = READ_ONLY_MEMBER_PATH + USER_NAME_PATH
RO_GROUP_NAME_PATH = READ_ONLY_MEMBER_PATH + GROUP_NAME_PATH
RW_USER_NAME_PATH = READ_WRITE_MEMBER_PATH + USER_NAME_PATH
RW_GROUP_NAME_PATH = READ_WRITE_MEMBER_PATH + GROUP_NAME_PATH
NAME_FMT = '"%s"'
GROUP_NAME_FMT = '"@%s"'
VALID_USERS_KEY = "valid users"
ADMIN_USERS_KEY = "admin users"
READ_LIST_KEY = "read list"
WRITE_LIST_KEY = "write list"

SMB_CONF_SECTION_FMT = "[%s]\n"
SMB_CONF_ITEM_FMT = "\t%s = %s\n"

RELOAD_OPT = "reload"

SEND_LOGEXT_BASE_ARGS = [SEND_LOGEXT_CMD, "--system"]
TO_SEND_LOGEXT_ARG_MAP_LIST = (
        "eventlog/log/@timestamp", 
        "eventlog/log/levelname", 
        "eventlog/log/category/name", 
        "eventlog/log/message", 
    )


class PalLogExt(palcmd.PalCmd):
    """Pal Migration command implementation class."""

    #
    # Internal methods
    #

    def _check_body_node(self, node, type=None):
        """Check if entry's body node is valid."""

        if node.get_node_name() != LOGEXT_NAME:
            raise InvalidArgError(node.get_node_name())

    def _transform_xml_to_dict(self, src_xml, map_info_list):
        """Transform "src_xml" XML data to dictionary."""

        transformed = {}
        for (key, target_xpath, default_value, converter) in map_info_list:
            if target_xpath:
                transformed[key] = \
                        src_xml.get_value(target_xpath, default_value)
            else:
                transformed[key] = default_value

            if callable(converter):
                transformed[key] = converter(transformed[key])

        return transformed

    def _get_name_list(self, share, name_xpath):
        """Return a name list gotten from node specified "name_xpath"."""

        return [NAME_FMT % name for name in share.get_value_list(name_xpath)
                         if name != ""]

    def _get_group_list(self, share, name_xpath):
        """Return a group name list gotten from node specified "name_xpath"."""

        return [GROUP_NAME_FMT % name for name in
                        share.get_value_list(name_xpath) if name != ""]

    def _get_share_value_setting(self, share, is_read_only):
        """Return a dictionaly include share settings."""

        def _convert_read_only(value):
            """Convert "value" for read only setting."""

            if is_read_only:
                return YES_VALUE

            if value.lower() == TRUE_TEXT:
                return YES_VALUE
            return NO_VALUE

        SETTING_MAP_INFO_LIST = (
                ("browseable", "", NO_VALUE, None),
                ("comment", "desc", "", None),
                ("path", "path", "", None),
                ("public", "", NO_VALUE, None),
                ("read only", "readonly/@enable", FALSE_TEXT,
                    _convert_read_only),
            )

        setting = self._transform_xml_to_dict(share, SETTING_MAP_INFO_LIST)

        ro_user_list = self._get_name_list(share, RO_USER_NAME_PATH)
        ro_group_list = self._get_group_list(share, RO_GROUP_NAME_PATH)
        rw_user_list = self._get_name_list(share, RW_USER_NAME_PATH)
        rw_group_list = self._get_group_list(share, RW_GROUP_NAME_PATH)

        setting[READ_LIST_KEY] = ",".join(ro_user_list + ro_group_list)
        setting[WRITE_LIST_KEY] = ",".join(rw_user_list + rw_group_list)
        setting[VALID_USERS_KEY] = ",".join(ro_user_list + ro_group_list +
                                            rw_user_list + rw_group_list)
        setting[ADMIN_USERS_KEY] = ""
        return setting

    def _create_temp_share_part_file(self, share, file_name, is_read_only):
        """Create a samba share part file temporarily."""

        share_name = share.get_node(NAME_PATH).get_text()
        tmp_path = os.path.join(config.TEMP_DIR,
                                file_name + os.extsep + str(os.getpid()))

        setting_dict = self._get_share_value_setting(share, is_read_only)
        with open(tmp_path, "w") as tmp_part:
            tmp_part.write(SMB_CONF_SECTION_FMT % share_name)
            for key, value in setting_dict.items():
                tmp_part.write(SMB_CONF_ITEM_FMT % (key, value))
            tmp_part.write("\n")
        return tmp_path

    def _create_samba_share_part_file(self, share):
        """Create a samba share part file for LOG Extention package."""

        conf_dir = LOGEXT_CONF_DIR
        if not os.path.exists(conf_dir):
            os.makedirs(conf_dir)

        for suffix in [RO_SUFFIX, RW_SUFFIX]:
            part_file = CONF_BASE_NAME + os.extsep + suffix
            part_path = os.path.join(conf_dir, part_file)
            tmp_path = self._create_temp_share_part_file(share, part_file,
                                                        suffix == RO_SUFFIX)
            shutil.move(tmp_path, part_path)

    def _daemon_reload(self):
        """Request a daemon to reload configuration."""

        try:
            cmd_args = [os.path.join(config.INITD_DIR, LOGEXTMGR_NAME),
                        RELOAD_OPT]
            CmdExec(cmd_args, stdin=NULL, stdout=NULL, stderr=NULL,
                    close_fds=True).execute()
        except:
            # Ignore error
            self.exception("reload %s fail" % LOGEXTMGR_NAME)

    def _create_link_for_samba(self):
        """Create symblic link files to share configuration file for samba."""

        conf_dir = LOGEXT_CONF_DIR
        conf_tmp_dir = LOGEXT_CONF_TEMP_DIR
        samba_dir = os.path.join(config.SMB_CONF_PART_DIR, config.INTERNAL_VOL)

        if not os.path.exists(conf_tmp_dir):
            os.makedirs(conf_tmp_dir)

        for suffix in [RO_SUFFIX, RW_SUFFIX]:
            part_file = CONF_BASE_NAME + os.extsep + suffix
            part_path = os.path.join(conf_dir, part_file)
            tmp_part_path = os.path.join(conf_tmp_dir, part_file)
            samba_part_path = os.path.join(samba_dir, part_file)

            for path in [tmp_part_path, samba_part_path]:
                try:
                    os.unlink(path)
                except:
                    # Ignore error
                    pass

            os.symlink(part_path, tmp_part_path)
            os.symlink(tmp_part_path, samba_part_path)

    def _delete_link_for_samba(self):
        """Delete symblic link files to share configuration file for samba."""

        conf_dir = LOGEXT_CONF_DIR
        conf_tmp_dir = LOGEXT_CONF_TEMP_DIR
        samba_dir = os.path.join(config.SMB_CONF_PART_DIR, config.INTERNAL_VOL)

        for suffix in [RO_SUFFIX, RW_SUFFIX]:
            part_file = CONF_BASE_NAME + os.extsep + suffix
            tmp_part_path = os.path.join(conf_tmp_dir, part_file)
            samba_part_path = os.path.join(samba_dir, part_file)

            for path in [tmp_part_path, samba_part_path]:
                if os.path.islink(path):
                    os.unlink(path)

    #
    # Request handlers
    #

    def _apply_handler(self, entry):
        """Handle apply request."""

        self._check_body_node(entry.get())

        try:
            try:
                share = entry.get().get_node(SHARE_NAME)
            except:
                # Don't update share setting
                pass
            else:
                self._create_samba_share_part_file(share)

            self._daemon_reload()
            
        except:
            self.exception("apply fail.")
            entry.set_result(Results.APPLY_ERROR)
            return entry

        entry.set_result(Results.OK)
        return entry

    def _enable_handler(self, entry):
        """Handle enable request."""

        self._check_body_node(entry.get())

        try:
            self._create_link_for_samba()
        except:
            self.exception("create link fail")
            entry.set_result(Results.CREATE_ERROR)
            return entry

        entry.set_result(Results.OK)
        return entry

    def _disable_handler(self, entry):
        """Handle disable request."""

        self._check_body_node(entry.get())

        try:
            self._delete_link_for_samba()
        except:
            self.exception("delete link fail")
            entry.set_result(Results.DELETE_ERROR)
            return entry

        entry.set_result(Results.OK)
        return entry

    def _send_handler(self, entry):
        """Handle send request."""

        self._check_body_node(entry.get())

        try:
            send_logext_args = list(SEND_LOGEXT_BASE_ARGS)
            for xpath in TO_SEND_LOGEXT_ARG_MAP_LIST:
                send_logext_args.append(entry.get().get_value(xpath))
            CmdExec(send_logext_args, stdin=NULL, stdout=NULL, stderr=NULL,
                    close_fds=True).execute()
        except:
            self.exception("send error")
            entry.set_result(Results.SEND_ERROR)
            return entry

        entry.set_result(Results.OK)
        return entry

    #
    # Constructor
    #

    def __init__(self, option=None):
        """Constructor."""

        palcmd.PalCmd.__init__(self, option=option)

        self.set_handler(ReqTypes.APPLY, self._apply_handler)
        self.set_handler(ReqTypes.ENABLE, self._enable_handler)
        self.set_handler(ReqTypes.DISABLE, self._disable_handler)
        self.set_handler(ReqTypes.SEND, self._send_handler)


#
# Main
#

if __name__ == "__main__":
    PalLogExt(option=palcmd.PalCmdOption()).handle_request()
