#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""Event handler for the eventlog in the logext package."""

import ionas.request as request
import libevh.evhcmd as evhcmd
import fal_msginfo as msginfo
from pal_info import (PAL_LOGEXT_PATH,)

from iobase.xmldata import XmlNode
from iobase.exception import (InvalidArgError, )
from product_conf.reqinfo import (Results, ReqTypes)
from pkgaddon.logext_config import (LEVEL_STR_MAP,)


EVENTINFO_NAME = "eventinfo"
CATEGORY_ATTR = "category"
ACTION_ATTR = "action"
PREVIOUS_ATTR = "previous"

EVENTLOG_CATEGORY = "eventlog"
ADD_ACTION = "add"

TIMESTAMP_ATTR = "timestamp"
LEVEL_ATTR = "level"
CODE_NAME = "code"
ARGLIST_NAME = "arglist"
ARG_NAME = "arg"
NAME_ATTR = "name"

LOG_PATH = "eventinfo/eventlog/log"
MESSAGEINFO_PATH = "eventinfo/eventlog/messages/messageinfolist/messageinfo"
MESSAGE_PATH = 'message'
ARGTYPE_PATH = 'argtypelist/argtype'

LOGEXT_LOG_XML_TMPL = """<logext>
    <eventlog>
        <log timestamp="%s">
            <levelname>%s</levelname>
            <category>
                <name>%s</name>
            </category>
            <message>%s</message>
        </log>
    </eventlog>
</logext>
"""

MESSAGE_XML_BASE = \
    "<message>" + \
    " <header>" + \
    "  <subject/>" + \
    " </header>" + \
    " <body/>" + \
    "</message>"


class EvhEventLog(evhcmd.EvhCmd):
    """Event Handler for the eventlog."""

    def __send_request(self, target, type, child_node=""):
        """Send request to pal module(single entry)"""

        entry = request.Entry(type, child_node)
        return self.send_entry(entry, target)

    def __get_log_level_str(self, log):
        """Return a string corresponding to a log level."""

        try:
            level = log.get_attr_value(LEVEL_ATTR)
        except:
            level = "0"

        return LEVEL_STR_MAP[level]

    def __get_category_str(self, log):
        """Return a string corresponding to a log category."""

        return msginfo.get_category_str(log.get_attr_value(CATEGORY_ATTR))

    def __get_sorted_arg_text_tuple(self, arglist, argtype_list):
        """Return the text list of element of the sorted "arg_list"
           based on the order of "argtype_list".
        """

        arg_list = arglist.get_node_list(ARG_NAME)
        if len(arg_list) < len(argtype_list):
            raise InvalidArgError("too few arg for message!!(" +
                str(len(arg_list)) + " < " + str(len(argtype_list)) + ")")

        sorted_list = []
        for argtype in argtype_list:
            try:
                sorted_list.append(arglist.get_node(ARG_NAME,
                                        {NAME_ATTR: argtype}).get_text())
            except:
                sorted_list.append("")

        return tuple(sorted_list)

    def __get_lang_value(self, node):
        """Return the value of "xml:lang" attribute of "node".
           empty string is returned if attribute doesn't exist.
        """

        if msginfo.XML_LANG_ATTR_NAME in node.get_attrs():
            return node.get_attrs()[msginfo.XML_LANG_ATTR_NAME]
        return ""

    def __get_message_str_for_lang(self, messageinfo, lang):
        """Return the message string corresponding to "lang" ."""

        default = ""
        for message in messageinfo.get_node_list(MESSAGE_PATH):
            lang_value = self.__get_lang_value(message)
            if len(lang_value) <= 0:
                default = message.get_text()

            if lang == lang_value:
                return message.get_text()

        return default

    def __get_message_str(self, log, messageinfo):
        """Return a string corresponding to a log code."""

        arglist = XmlNode(ARGLIST_NAME)
        try:
            arglist = log.get_node(ARGLIST_NAME)
        except:
            # No arguments
            pass
        message_base = self.__get_message_str_for_lang(messageinfo,
                                                        msginfo.get_lang())
        try:
            argtype_list = messageinfo.get_value_list(ARGTYPE_PATH)
        except:
            return message_base

        return message_base % \
                self.__get_sorted_arg_text_tuple(arglist, argtype_list)

    def __handle_eventlog_add_event(self, eventinfo, entry):
        """Handle eventlog:add event."""

        if eventinfo.is_set_to_true(PREVIOUS_ATTR):
            return

        log = entry.get().get_node(LOG_PATH)
        messageinfo = entry.get().get_node(MESSAGEINFO_PATH)

        body_xml = LOGEXT_LOG_XML_TMPL % (log.get_attr_value(TIMESTAMP_ATTR),
                                self.__get_log_level_str(log),
                                self.__get_category_str(log),
                                self.__get_message_str(log, messageinfo))

        self.__send_request(PAL_LOGEXT_PATH, ReqTypes.SEND, body_xml)

    def __handle_eventlog_event(self, eventinfo, entry):
        """Handle eventlog event."""

        action = eventinfo.get_attr_value(ACTION_ATTR)

        if action == ADD_ACTION:
            self.__handle_eventlog_add_event(eventinfo, entry)

        entry.set_result(Results.OK)
        return entry

    #
    # Request handlers
    #

    def __event_handler(self, entry):
        """Handle "event" request."""

        try:
            eventinfo = entry.get().get_node(EVENTINFO_NAME)
            if eventinfo.get_attr_value(CATEGORY_ATTR) == EVENTLOG_CATEGORY:
                return self.__handle_eventlog_event(eventinfo, entry)
        except:
            # Ignore Error
            self.exception("logext")

        entry.set_result(Results.OK)
        return entry

    def __init__(self, option=None):
        """Constructor."""

        evhcmd.EvhCmd.__init__(self, option=option)

        self.set_handler(ReqTypes.EVENT, self.__event_handler)

#
# Main
#

if __name__ == "__main__":
    EvhEventLog(option=evhcmd.EvhCmdOption()).handle_request()
