#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""Do something for uninstalling logext."""

import os
import sys
import shutil

import iobase.xmldata as xmldata
import iobase.cmdexec as cmdexec
import ionas.request as request
from product_conf import (LIBEXEC_DIR, STOR_IF_TO_VOL, )
from product_conf.devinfo import (VOL_INFO, )
from fal_info import (FAL_VOLCTRL_PATH, FAL_SERVICECTRL_PATH,
                        FAL_LOGEXT_PATH, Results, ReqTypes)
from pal_info import (PAL_SAMBA_PATH, PAL_LOGEXT_PATH)


INTERNAL_VOL = STOR_IF_TO_VOL["internal"][1]
MANUAL_PATH = os.path.join(VOL_INFO[INTERNAL_VOL]["mnt_point"],
                            VOL_INFO[INTERNAL_VOL]["share_root"],
                            "LAN DISK Manual",
                            "ログ拡張パッケージ_画面で見るマニュアル.pdf")

UNMOUNT_EVENT_XML = '<events><eventinfo category="volume" ' + \
                                'action="unmount" previous="true">' + \
                    '<volume><name>' + INTERNAL_VOL + '</name></volume>' + \
                    '</eventinfo></events>'

VOLUME_QUERY_REQ_XML_TMPL = \
    '<volctrl><volume><name type="volume">%s</name></volume></volctrl>'

VOLUME_STATUS_PATH = "volume/status"
STATUS_MOUNTED = "mounted"

LOGEXT_NAME = "logext"
SAMBA_NAME = "samba"
SHARE_PATH_PATH = "share/path"
SERVICECTRL_SHARE_REQ_XML = """<servicectrl>
    <categorylist>
        <category><name>share</name></category>
    </categorylist>
</servicectrl>
"""
RESTRICTION_LOGSHARE_XPATH = \
        """restrictionlist/restriction[@type="logshare"]"""


def __send_request(target, type, body=""):
    """Send request entry to "target"."""

    entry = request.Entry(type, body)
    return request.Request(target).request(entry).get_entry()


def is_volume_mounted(vol):
    """Check if "vol" is mounted."""

    result_entry = __send_request(FAL_VOLCTRL_PATH, ReqTypes.QUERY,
                            VOLUME_QUERY_REQ_XML_TMPL % (vol,))

    if (result_entry.get_result() == Results.OK and
        result_entry.get().get_value(VOLUME_STATUS_PATH) == STATUS_MOUNTED):

        return True

    return False


def uninstall_manual():
    """Uninstall manual for the fxwf package by a event handler."""

    if os.path.exists(MANUAL_PATH):
        os.unlink(MANUAL_PATH)


def load_config():
    """Load configuration file."""

    return __send_request(FAL_LOGEXT_PATH, ReqTypes.LOAD).get()


def disable_logext_share(config):
    """Disable logext share."""

    __send_request(FAL_SERVICECTRL_PATH, ReqTypes.SUSPEND,
                    SERVICECTRL_SHARE_REQ_XML)

    # To force disable logext share, using PAL command.
    __send_request(PAL_LOGEXT_PATH, ReqTypes.DISABLE, LOGEXT_NAME)
    __send_request(PAL_SAMBA_PATH, ReqTypes.UPDATE, SAMBA_NAME)

    __send_request(FAL_SERVICECTRL_PATH, ReqTypes.RESUME,
                    SERVICECTRL_SHARE_REQ_XML)


def delete_log_directory(config):
    """Create a Log directory."""

    share_path = config.get_node(SHARE_PATH_PATH).get_text()

    if os.path.exists(share_path):
        shutil.rmtree(share_path)


#
# Main
#

if not is_volume_mounted(INTERNAL_VOL):
    sys.exit(1)

uninstall_manual()

config = load_config()
delete_log_directory(config)
disable_logext_share(config)
