# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from django import forms
from django.utils.translation import ugettext as _
from project.forms.fields.checkbox import CheckboxField
from project.forms.fields.extras.enable import EnableField
from project.forms.fields.extras.user import MultiSelectUserField
from project.forms.fields.extras.group import MultiSelectGroupField
from django.forms import IntegerField
import math


##############################################################################
#
from django import forms
from itertools import chain
from django.utils.encoding import force_text, python_2_unicode_compatible
from django.utils import six
from django.utils.safestring import SafeData, mark_safe
from django.utils.functional import allow_lazy


class RendererMixin(object):
    renderer = None  # subclasses must define this
    _empty_value = None

    def __init__(self, *args, **kwargs):
        # Override the default renderer if we were passed one.
        renderer = kwargs.pop('renderer', None)
        if renderer:
            self.renderer = renderer
        super(RendererMixin, self).__init__(*args, **kwargs)

    def subwidgets(self, name, value, attrs=None, choices=()):
        for widget in self.get_renderer(name, value, attrs, choices):
            yield widget

    def get_renderer(self, name, value, attrs=None, choices=()):
        """Returns an instance of the renderer."""
        if value is None:
            value = self._empty_value
        final_attrs = self.build_attrs(attrs)
        choices = list(chain(self.choices, choices))
        return self.renderer(name, value, final_attrs, choices)

    def render(self, name, value, attrs=None, choices=()):
        return self.get_renderer(name, value, attrs, choices).render()

    def id_for_label(self, id_):
        # Widgets using this RendererMixin are made of a collection of
        # subwidgets, each with their own <label>, and distinct ID.
        # The IDs are made distinct by y "_X" suffix, where X is the zero-based
        # index of the choice field. Thus, the label for the main widget should
        # reference the first subwidget, hence the "_0" suffix.
        if id_:
            id_ += '_0'
        return id_


def escape(text):
    """
    Returns the given text with ampersands, quotes and angle brackets encoded for use in HTML.
    """
    return mark_safe(force_text(text).replace('&', '&amp;').replace('<', '&lt;').replace('>', '&gt;').replace('"', '&quot;').replace("'", '&#39;'))
escape = allow_lazy(escape, six.text_type)


def conditional_escape(text):
    """
    Similar to escape(), except that it doesn't operate on pre-escaped strings.
    """
    if isinstance(text, SafeData):
        return text
    else:
        return escape(text)


def format_html(format_string, *args, **kwargs):
    """
    Similar to str.format, but passes all arguments through conditional_escape,
    and calls 'mark_safe' on the result. This function should be used instead
    of str.format or % interpolation to build up small HTML fragments.
    """
    args_safe = map(conditional_escape, args)
    kwargs_safe = dict([(k, conditional_escape(v)) for (k, v) in
                        six.iteritems(kwargs)])
    return mark_safe(format_string.format(*args_safe, **kwargs_safe))


def format_html_join(sep, format_string, args_generator):
    """
    A wrapper of format_html, for the common case of a group of arguments that
    need to be formatted using the same format string, and then joined using
    'sep'. 'sep' is also passed through conditional_escape.

    'args_generator' should be an iterator that returns the sequence of 'args'
    that will be passed to format_html.

    Example:

      format_html_join('\n', "<li>{0} {1}</li>", ((u.first_name, u.last_name)
                                                  for u in users))

    """
    return mark_safe(conditional_escape(sep).join(
            format_html(format_string, *tuple(args))
            for args in args_generator))


def flatatt(attrs):
    """
    Convert a dictionary of attributes to a single string.
    The returned string will contain a leading space followed by key="value",
    XML-style pairs.  It is assumed that the keys do not need to be XML-escaped.
    If the passed dictionary is empty, then return an empty string.

    The result is passed through 'mark_safe'.
    """
    return format_html_join('', ' {0}="{1}"', sorted(attrs.items()))


@python_2_unicode_compatible
class ChoiceFieldRenderer(object):
    """
    An object used by RadioSelect to enable customization of radio widgets.
    """

    choice_input_class = None

    def __init__(self, name, value, attrs, choices):
        self.name = name
        self.value = value
        self.attrs = attrs
        self.choices = choices

    def __iter__(self):
        for i, choice in enumerate(self.choices):
            yield self.choice_input_class(self.name, self.value, self.attrs.copy(), choice, i)

    def __getitem__(self, idx):
        choice = self.choices[idx] # Let the IndexError propogate
        return self.choice_input_class(self.name, self.value, self.attrs.copy(), choice, idx)

    def __str__(self):
        return self.render()

    def render(self):
        """
        Outputs a <ul> for this set of choice fields.
        If an id was given to the field, it is applied to the <ul> (each
        item in the list will get an id of `$id_$i`).
        """
        id_ = self.attrs.get('id', None)
        #start_tag = format_html('<ul id="{0}">', id_) if id_ else '<ul>'
        start_tag = format_html(mark_safe('<ul id="{0}">'), id_) if id_ else '<ul>'
        output = [start_tag]
        for widget in self:
            #output.append(format_html('<li>{0}</li>', force_text(widget)))
            output.append(format_html(mark_safe(u'<li>{0}</li>'), force_text(widget)))
        output.append('</ul>')
        return mark_safe('\n'.join(output))


@python_2_unicode_compatible
class SubWidget(object):
    """
    Some widgets are made of multiple HTML elements -- namely, RadioSelect.
    This is a class that represents the "inner" HTML element of a widget.
    """
    def __init__(self, parent_widget, name, value, attrs, choices):
        self.parent_widget = parent_widget
        self.name, self.value = name, value
        self.attrs, self.choices = attrs, choices

    def __str__(self):
        args = [self.name, self.value, self.attrs]
        if self.choices:
            args.append(self.choices)
        return self.parent_widget.render(*args)


@python_2_unicode_compatible
class ChoiceInput(SubWidget):
    """
    An object used by ChoiceFieldRenderer that represents a single
    <input type='$input_type'>.
    """
    input_type = None  # Subclasses must define this

    def __init__(self, name, value, attrs, choice, index):
        self.name = name
        self.value = value
        self.attrs = attrs
        self.choice_value = force_text(choice[0])
        self.choice_label = force_text(choice[1])
        self.index = index
        if 'id' not in self.attrs:  # NOTE 改造
            self.attrs['id'] = 'id_%s' % name

    def __str__(self):
        return self.render()

    def render(self, name=None, value=None, attrs=None, choices=()):
        name = name or self.name
        value = value or self.value
        attrs = attrs or self.attrs
        if 'id' in self.attrs:
            ##label_for = format_html(' for="{0}_{1}"', self.attrs['id'], self.index)
            #label_for = format_html(mark_safe(' for="{0}_{1}"'), self.attrs['id'], self.index)
            label_for = format_html(mark_safe(' for="{0}"'), self.auto_id())
        else:
            label_for = ''
        #return format_html('<label{0}>{1} {2}</label>', label_for, self.tag(), self.choice_label)
        return format_html(mark_safe(u'<label{0}>{1} {2}</label>'), label_for, self.tag(), self.choice_label)

    def is_checked(self):
        return self.value == self.choice_value

    def tag(self):
        #if 'id' in self.attrs:
        #    self.attrs['id'] = '%s_%s' % (self.attrs['id'], self.index)
        #final_attrs = dict(self.attrs, type=self.input_type, name=self.name, value=self.choice_value)
        final_attrs = dict(self.attrs, type=self.input_type, name=self.name, value=self.choice_value, id=self.auto_id())
        if self.is_checked():
            final_attrs['checked'] = 'checked'
        #return format_html('<input{0} />', flatatt(final_attrs))
        return format_html(mark_safe('<input{0} />'), flatatt(final_attrs))

    def auto_id(self):
        return '%s_%s' % (self.attrs['id'], self.index)


class CheckboxChoiceInput(ChoiceInput):
    input_type = 'checkbox'

    def __init__(self, *args, **kwargs):
        super(CheckboxChoiceInput, self).__init__(*args, **kwargs)
        self.value = set(force_text(v) for v in self.value)

    def is_checked(self):
        return self.choice_value in self.value


class CheckboxFieldRenderer(ChoiceFieldRenderer):
    choice_input_class = CheckboxChoiceInput


#class CheckboxSelectMultiple(RendererMixin, SelectMultiple):
#    renderer = CheckboxFieldRenderer
#    _empty_value = []


class CheckboxSelectMultiple(RendererMixin, forms.CheckboxSelectMultiple):
    renderer = CheckboxFieldRenderer
    _empty_value = []
                                                                             #
##############################################################################


class KeylistField(forms.MultipleChoiceField):
    widget = CheckboxSelectMultiple
    default_validators = []
    default_error_messages = {
#        'required': _(u'This field is required.'),
#        'invalid': _(u'Enter a valid value.'),
#        'invalid_choice': _(u'Select a valid choice. %(value)s is not one of the available choices.'),
#        'invalid_list': _(u'Enter a list of values.'),
    }

    def __init__(self, attrs={}, *args, **kwargs):
        if 'choices' not in kwargs:
            kwargs['choices'] = self._default_choices()
        if 'widget' not in kwargs:
            kwargs['widget'] = self.widget(attrs=attrs)
        if 'required' not in kwargs:
            kwargs['required'] = False
        super(KeylistField, self).__init__(*args, **kwargs)

    def _default_choices(self):
        try:
            return [("PID0,VID0,SERIAL0", "0"), ("PID1,VID1,SERIAL1", "1")]
        except:
            return []
